@@ -21,7 +21,7 @@ quad2trimesh <- function(mesh, updateNormals=TRUE) {
         warning("this is no quadmesh, nothing to be done")
     } else {
         ib2it <- cbind(mesh$ib[1:3,,drop=FALSE],mesh$ib[c(3:4,1),,drop=FALSE])
-        mesh$it <- ib2it
+        mesh$it <- cbind(mesh$it,ib2it)
         mesh$ib <- NULL
         if (updateNormals) {
             mesh <- vcgUpdateNormals(mesh)
